// The contents of this file are subject to the Mozilla Public License
// Version 1.1 (the "License"); you may not use this file except in
// compliance with the License.  You may obtain a copy of the License
// at http://www.mozilla.org/MPL/
//
// Software distributed under the License is distributed on an "AS IS"
// basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.
// See the License for the specific language governing rights and
// limitations under the License.
//
// The Original Code is
// "Michael C. Lee, Jr's small example of the many features of Java3D".
//
// The Initial Developer of the Original Code is GameSpawn, Inc.
// Portions created by GameSpawn, Inc. are
// Copyright (C) 2000 GameSpawn, Inc.  All Rights Reserved.
//
// Contributor(s): 

package com.gamespawn.lab.mikelee.j3d.examples;

import java.awt.*;
import java.applet.Applet;
import com.sun.j3d.utils.applet.MainFrame;
import com.sun.j3d.utils.universe.*;
import com.sun.j3d.utils.geometry.*;
import javax.media.j3d.*;
import javax.vecmath.*;
import com.gamespawn.lab.mikelee.j3d.geometry.*;

/**
* Demonstrates some of the basic features of Java3D. Mainly geometry, basic scenes and lighting.
*
* @author
*   Michael C. Lee, Jr. (mike@thoselees.com, http://www.thoselees.com/mike)
*
* @version Java 1.2, Java3d 1.1.3
*/
public class BasicGeometry2 extends Applet
{
    public BasicGeometry2()
    {
        // set up window
        setLayout(new BorderLayout());
        Canvas3D c = new Canvas3D(null);
        add("Center", c);

        // Create a simple scene and attach it to the virtual universe
        BranchGroup scene = createSceneGraph(c);
        SimpleUniverse u = new SimpleUniverse(c);

        // This will move the ViewPlatform back a bit so the objects in the scene can be viewed.
        u.getViewingPlatform().setNominalViewingTransform();

        // Create shpere
        Diamond diamond = new Diamond();
//        diamond.setPrimitiveFlags(Primitive.GENERATE_NORMALS);

        // Set location of sphere
        Transform3D t = new Transform3D();
      	t.set(0.5, new Vector3d(0.0f, 0.0f, -2.0f));
        TransformGroup objTrans = new TransformGroup(t);
        objTrans.addChild(diamond);
        scene.addChild(objTrans);

       	u.addBranchGraph(scene);
    }

    /**
    * Creates a BranchGroup with the appropriate lighting for our scene.
    *
    * @param canvas The component which the 3d scene will be rendered onto.
    * @return A branch group containing the lighting for our scene.
    */
    public BranchGroup createSceneGraph(Canvas3D canvas)
    {
        BranchGroup objRoot = new BranchGroup();

    	// Create a bounds for the lights
        BoundingSphere bounds = new BoundingSphere(new Point3d(0.0,0.0,0.0), 100.0);

    	// Set up the global lights
    	Color3f color = new Color3f(1.0f, 0.0f, 0.0f);
        Vector3f dir = new Vector3f(-1.0f, 1.0f, -1.0f);

        DirectionalLight light = new DirectionalLight(color, dir);
    	light.setInfluencingBounds(bounds);
    	objRoot.addChild(light);

        return objRoot;
    }

    public static void main(String[] args)
    {
        new MainFrame(new BasicGeometry2(), 700, 700);
    }
}
